"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const expandBraces = require("brace-expansion");
const isNegatedGlob = require("is-negated-glob");
const path = require("path");
/**
 * @param input the glob pattern with forward slash as directory separator
 * @param cwd the absolute path of the current directory with forward slash or backslash as directory separator.
 */
function* normalizeGlob(input, cwd) {
    if (input === '')
        input = '.';
    for (let pattern of expandBraces(input)) {
        const trailingSlash = pattern.endsWith('/');
        const ing = isNegatedGlob(pattern);
        ({ pattern } = ing);
        let { root } = path.parse(pattern);
        let parts;
        if (root) {
            parts = pattern.substr(root.length).split(/\/+/g);
        }
        else {
            ({ root } = path.parse(cwd));
            if (!root)
                throw new Error("'cwd' must be an absolute path");
            parts = [...cwd.substr(root.length).split(/[/\\]+/g), ...pattern.split(/\/+/g)];
            root = ensureTrailingSlash(root.replace(/\\$/, ''));
        }
        if (ing.negated)
            root = '!' + root;
        yield* appendPath([], parts, 0, root, trailingSlash);
    }
}
exports.normalizeGlob = normalizeGlob;
function* appendPath(result, parts, i, prefix, trailingSlash) {
    for (; i < parts.length; ++i) {
        switch (parts[i]) {
            case '':
            case '.':
                break;
            case '..':
                if (result.pop() === '**') {
                    if (i === parts.length - 1)
                        trailingSlash = true;
                    yield* appendPath(result.slice(0, -1), parts, i + 1, prefix, trailingSlash);
                    result.push('**');
                }
                break;
            case '**':
                // collapse multiple globstars into one
                if (result[result.length - 1] === '**')
                    break;
            // falls through
            default:
                result.push(parts[i]);
        }
    }
    const joined = prefix + result.join('/');
    yield trailingSlash ? ensureTrailingSlash(joined) : joined;
}
function ensureTrailingSlash(p) {
    return p.endsWith('/') ? p : p + '/';
}
//# sourceMappingURL=index.js.map